@@ -1,4 +1,3 @@
-import json
 import logging
 import os.path
 import time
@@ -6,10 +5,9 @@ from typing import List, Optional, Any, TYPE_CHECKING
 
 from s3transfer.constants import GB
 from tqdm import tqdm
-from urllib.parse import urlparse
 
-from metaloop.client.cloud_storage import CloudConfig, CloudClient, Item
-from metaloop.client.download_helper import DownloadHelper
+from metaloop.client.cloud_storage import CloudClient
+from metaloop.dataset.exporter import Exporter
 from metaloop.utils.file_helper import *
 from metaloop.exception import ResourceNotExistError, InvalidParamsError, InternalServerError
 
@@ -118,6 +116,10 @@ class Dataset(DatasetMeta):
     def id(self):
         return self._id
 
+    @property
+    def annotation_status(self):
+        return self._annotation_status
+
     def create_version(
             self,
             inherited_version_number: Optional[int] = None,
@@ -234,7 +236,7 @@ class Dataset(DatasetMeta):
             index = self._data_count + index
         if index >= self._data_count:
             raise IndexError
-        if index >= self.iter_current_start and index < self.iter_current_end:
+        if self.iter_current_start <= index < self.iter_current_end:
             return self.iter_current_object[index - self.iter_current_start]
         try:
             response = self._x_api.list_objects(self._id, index, self.iter_batch_size)
@@ -277,8 +279,8 @@ class Dataset(DatasetMeta):
         if storage_type not in {"local", "cos"}:
             raise InvalidParamsError(param_name="storage_type", param_value=storage_type)
 
-        if append_path:
-            append_path = self._x_api.get_dataset_path(self._id, append_path)
+        # if append_path:
+        #     append_path = self._x_api.get_dataset_path(self._id, append_path)
 
         print(f"upload file to server, please wait...")
 
@@ -334,7 +336,8 @@ class Dataset(DatasetMeta):
             self,
             file_path: str,
             storage_type: Optional[str] = "local",
-            export_path: Optional[List[str]] = None
+            export_path: Optional[List[str]] = None,
+            dataset_ids: Optional[List[str]] = None
     ) -> None:
         """Export data to local file from dataset.
 
@@ -346,30 +349,35 @@ class Dataset(DatasetMeta):
                 2. "external": Tencent Cloud Object Storage (COS).
 
             export_path: Export data from the specific path[s].
+            dataset_ids: IDs of dataset to export in batch.
 
         """
         if storage_type not in {"local", "external"}:
             raise InvalidParamsError(param_name="storage_type", param_value=storage_type)
 
         if storage_type == "external":
-            self._get_cloud_storage_config('', storage_type=storage_type)
+            CloudClient.get_cloud_storage_config(self._x_api, '', storage_type=storage_type)
+
         self.checkout(self._version)
         export_type = "annotation_uri" if self._annotation_status == 2 else "uri_json_standard"
 
         post_data: Dict[str, Any] = {
-            "dataset_id": self._id,
+            "dataset_id": self._id if not dataset_ids else dataset_ids,
             "export_type": export_type,
             "storage_type": storage_type,
             "task_type": "shared_links",
         }
         if export_path:
             if isinstance(export_path, list):
-                path = [self._x_api.get_dataset_path(self._id, path) for path in export_path]
-                post_data["export_path"] = path
+                # path = [self._x_api.get_dataset_path(self._id, path) for path in export_path]
+                post_data["export_path"] = export_path
             else:
                 raise InvalidParamsError(message=f"export_path value type should be list")
 
-        self._x_api.post_export(self._id, post_data)
+        if not dataset_ids:
+            self._x_api.post_export(self._id, post_data)
+        else:
+            self._x_api.post_export_multi(self._id, post_data)
         print("export data start, please wait...")
 
         with tqdm(total=100) as pbar:
@@ -380,62 +388,8 @@ class Dataset(DatasetMeta):
         print("download data from server, please wait...")
         time.sleep(0.3)
 
-        if not os.path.exists(file_path):
-            os.makedirs(file_path, 0o0775)
-
-        file_writer = open(os.path.join(file_path, "output.json.tmp"), "w")
-        download_helper = DownloadHelper(self._x_api)
-
-        count = 0
-        for obj in catalog:
-            count += 1
-            ex = False
-            image_path = obj["image_path"] if "image_path" in obj else ""
-
-            if "external_url_image" in obj:
-                # /{bucket}/{path}/{to}/{object}.ext
-                url = obj["external_url_image"]
-
-                items = url.strip("/").split("/")
-                if len(items) < 2:
-                    continue
-
-                s3_config = self._get_cloud_storage_config(items[0], storage_type=storage_type)
-
-                file_name = download_helper.download_s3_file(
-                    self._download_file,
-                    identifier=s3_config.identifier,
-                    bucket=items[0],
-                    url=url,
-                    file_path=file_path,
-                    image_path=image_path
-                )
-                obj["url_image"] = file_name
-                ex = True
-
-            if not ex:
-                # {endpoint}/{bucket}/{path}/{to}/{object}.ext
-                url = str(obj["url_image"])
-                file_name = download_helper.download_http_file(
-                    self._download_http_file,
-                    url=url,
-                    file_path=file_path,
-                    image_path=image_path
-                )
-                obj["url_image"] = file_name
-
-            body = json.dumps(obj, ensure_ascii=False)
-            file_writer.write(body)
-            file_writer.write("\n")
-            if not count % 128:
-                file_writer.flush()
-
-        download_helper.force_submit()
-        download_helper.wait()
-
-        file_writer.flush()
-        file_writer.close()
-        os.rename(os.path.join(file_path, "output.json.tmp"), os.path.join(file_path, "output.json"))
+        exporter = Exporter(self._x_api)
+        exporter.get_data(catalog, file_path, storage_type)
 
     def merge(
             self,
@@ -452,21 +406,6 @@ class Dataset(DatasetMeta):
         self._x_api.merge_dataset(self._id, dataset_ids)
         self.checkout(self._version)
 
-    def _get_dataset(
-            self,
-            dataset_id: str
-    ) -> Dict[str, Any]:
-        if not dataset_id:
-            raise ResourceNotExistError(resource="dataset", identification=dataset_id)
-
-        try:
-            response = self._x_api.get_version(dataset_id)
-            info = response.json()["data"][0]
-        except IndexError as error:
-            raise ResourceNotExistError(resource="dataset", identification=dataset_id) from error
-
-        return info
-
     def _inherit_import(
             self,
             dataset_id: str,
@@ -533,9 +472,14 @@ class Dataset(DatasetMeta):
             upload_prefix: Optional[str] = ""
     ) -> str:
         bucket = self._get_bucket(storage_type)
-        s3_config = self._get_cloud_storage_config("", storage_type)
+        s3_config = CloudClient.get_cloud_storage_config(self._x_api, "", storage_type)
+
         if not bucket:
             bucket = s3_config.default_bucket
+
+        if not upload_prefix:
+            upload_prefix = os.path.join("raw", time.strftime("%Y%m%d%H%M%S", time.localtime()))
+
         return CloudClient.upload_file(s3_config.identifier, bucket, file_path, upload_prefix)
 
     def _upload_files(
@@ -544,95 +488,18 @@ class Dataset(DatasetMeta):
             storage_type: str,
             upload_prefix: Optional[str] = ""
     ) -> str:
-        s3_config = self._get_cloud_storage_config("", storage_type)
+        s3_config = CloudClient.get_cloud_storage_config(self._x_api, "", storage_type)
 
         bucket = self._get_bucket(storage_type)
         if not bucket:
             bucket = s3_config.default_bucket
 
+        if not upload_prefix:
+            date_time = time.strftime("%Y%m%d", time.localtime())
+            upload_prefix = f"raw/{date_time}"
+
         return CloudClient.upload_files_parallel(s3_config.identifier, bucket, file_path, upload_prefix)
 
-    @staticmethod
-    def _download_file(
-            **kwargs
-    ) -> Tuple[str, Item]:
-        key, file_name, obj_url = Dataset._get_download_info(
-            kwargs["url"],
-            kwargs["file_path"],
-            kwargs["image_path"]
-        )
-        return kwargs["identifier"], Item(kwargs["bucket"], key, file_name, obj_url)
-
-    @staticmethod
-    def _download_http_file(
-            **kwargs
-    ) -> Item:
-        _, file_name, obj_url = Dataset._get_download_info(
-            urlparse(kwargs["url"]).path,
-            kwargs["file_path"],
-            kwargs["image_path"]
-        )
-        return Item("", kwargs["url"], file_name, obj_url)
-
-    @staticmethod
-    def _get_download_info(
-            url: str,
-            file_path: str,
-            image_path: str
-    ) -> Tuple[str, str, str]:
-        _, key = get_bucket_and_key(url)
-
-        if not image_path:
-            image_path = get_user_origin_path(key)
-
-        if not file_path or image_path.find("/") < 0:
-            image_path = os.path.join("abaddon", image_path)
-
-        file_name = os.path.join(file_path, image_path.strip("/"))
-
-        if not os.path.exists(os.path.dirname(file_name)):
-            os.makedirs(os.path.dirname(file_name), 0o0755)
-
-        while file_path.endswith('/'):
-            file_path = file_path[:-1]
-
-        obj_url = file_name[len(file_path) + 1:]
-
-        return key, file_name, obj_url
-
-    def _download_files(
-            self,
-            bucket: str,
-            prefix: str,
-            file_path: str
-    ) -> str:
-        s3_config = self._get_cloud_storage_config(bucket)
-        return CloudClient.download_files_parallel(s3_config.identifier, bucket, prefix, file_path)
-
-    def _get_cloud_storage_config(
-            self,
-            name: str,
-            storage_type: Optional[str] = ""
-    ) -> CloudConfig:
-        if not name:
-            name = storage_type
-
-        if not CloudClient.find_s3_config(name):
-            info = self._x_api.get_authorized_s3_config(name, storage_type)
-            identifier = info["name"]
-            endpoint = info["endpoint"]
-            access_key = info["access_key"]
-            secret_key = info["secret_key"]
-            default_bucket = info["bucket"]
-
-            CloudClient.set_s3_config(identifier, endpoint, access_key, secret_key, default_bucket)
-            if name != default_bucket:
-                CloudClient.set_s3_config(default_bucket, endpoint, access_key, secret_key, default_bucket)
-        else:
-            identifier = name
-
-        return CloudClient.find_s3_config(identifier)
-
     def _get_bucket(
             self,
             storage_type: str
@@ -663,28 +530,3 @@ class Dataset(DatasetMeta):
             upload_id = self._x_api.post_multipart_formdata(post_data)
 
         return upload_id
-
-    def _get_stream_data(
-            self,
-            url: str,
-            file_path: str
-    ) -> str:
-        if url.startswith("http://"):
-            file_name = url[url.index("http://") + 7:]
-        elif url.startswith("https://"):
-            file_name = url[url.index("https://") + 8:]
-        else:
-            file_name = url
-
-        index = file_name.find("/")
-        if index > -1:
-            file_name = os.path.join(file_path, file_name[index + 1:])
-        else:
-            file_name = os.path.join(file_path, file_name)
-
-        if not os.path.exists(os.path.dirname(file_name)):
-            os.makedirs(os.path.dirname(file_name), 0o0755)
-
-        self._x_api.get_stream_data(url, file_name)
-
-        return url[index + 8:]
